import java.awt.TextArea;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Random;
import javax.swing.JLabel;
import javax.swing.JList;

public class RFIDAP1GenerlizedTag {
    
    private Socket fclientSocket; 
    private OutputStreamWriter fout;
    private BufferedReader fin;
    private String fhost;
    private int fport;

    private int numOfEntries;
    private int n;
    private int i;
    public int[] ARV;   
    private int [] indices;
    private int [] reducedLRV;       
    private int keyEntry;             
    private int[] Y;                       
    
    private boolean[] isXored;
    
    private Thread t;
    private int fdelayFactor;
    private int fdisplayType;
    private int fdisplayBitNumber;
    
    private TextArea ftext;
    private JList fARVList;
    private JList fLRVList;
    private JList fYList;
    private JLabel fkeyEntryLabel;
    private JLabel fMsgLabel;
    
    

    // IntListUpdater class
    class IntListUpdater implements Command {
        int[] farray;
        JList flist;
        
        public IntListUpdater(int[] array, JList list) {
            farray = array;
            flist = list;
        }
        
        public void doMy() {
            if (farray != null && flist != null)
                RFIDCustom.displayArray(flist, farray, fdisplayType, fdisplayBitNumber);
         }        
    }
    
    class StrLabelUndater implements Command {

        JLabel flabel;
        String fstr;
        
        public StrLabelUndater(String str, JLabel label) {
            flabel = label;
            fstr = str;
        }
        
        public void doMy() {
            if (fstr != null && flabel != null)
                flabel.setText(fstr);
        }
        
    }
    
    // TagInit class
    class TagInit implements Runnable {
       
        public void run() {
            // Trying to connect to a socket and initialize an output stream
            try {
               fclientSocket = new Socket(fhost, fport); // host and port            
               //translate each character according to UTF-8.
               fout = new OutputStreamWriter(fclientSocket.getOutputStream(), "UTF-8");
            } catch (UnknownHostException e) {
                System.exit(1);
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }

            // Initialize an input stream
            try {
                fin = new BufferedReader(new InputStreamReader(fclientSocket.getInputStream()));
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }       

            ftext.append("Connected to server!\n");        
        }
    
    }//TagInit 
    
    public RFIDAP1GenerlizedTag(String host, int port, int[] FARV,
            int delayFactor, int displayType, int displayBitNumber,
            TextArea text,
            JList ARVList, JList LRVList, JList YList,
            JLabel keyEntryLabel,
            JLabel MsgLabel
            ) {
        fclientSocket = null; 
        fout = null;
        fin = null;
        fhost = host;
	fport = port;
        
        ftext = text;
        fdelayFactor = delayFactor;
        fdisplayType = displayType;
        fdisplayBitNumber = displayBitNumber;
        
        fARVList = ARVList;
        fLRVList = LRVList;
        fYList = YList;
        fkeyEntryLabel = keyEntryLabel;
        fMsgLabel = MsgLabel;
        
	i = 1;
	ARV = FARV.clone();
	n = ARV.length;
        //numOfEntries = (int)Math.round(Math.log(n)/Math.log(2));
        numOfEntries = (int)Math.round(2*Math.log(n));
	indices = new int[numOfEntries];
        for (int i=0; i<indices.length; i++) {
            indices[i] = -1;
        }
        reducedLRV = new int[numOfEntries];
        for (int i=0; i<reducedLRV.length; i++) {
            reducedLRV[i] = 0;
        }
        /**/isXored = new boolean[n];
        /**/resetIsXored();
        
        RFIDCustom.displayArray(fARVList, ARV, fdisplayType, fdisplayBitNumber);
        RFIDCustom.displayArray(fLRVList, reducedLRV, fdisplayType, fdisplayBitNumber);

        t = new Thread(new TagInit());
        t.start();
    }
    
    public void updateDisplay() {
        Command[] cs = {new IntListUpdater(ARV, fARVList),
                        new StrLabelUndater("" + keyEntry, fkeyEntryLabel),
                        new IntListUpdater(reducedLRV, fLRVList),
                        new IntListUpdater(Y, fYList),                        
                        };
        RFIDDisplayDeamon dd = new RFIDDisplayDeamon(cs, fdelayFactor);
        dd.start();
    }
    
    public void send(String msg) {
        String ans = null;
        
        // if msg is "open" then encrypt
        if (msg.equals(RFIDCustom.kwOpen)) {
            int[] Y = encrypt();
	    msg = String.valueOf(Y.length);
	    for (int i=0; i<Y.length; i++) {
	    	msg = msg + ";" + Y[i];
            }
        }
        
        try {
            // send the message
            fout.write(msg + "\n");
            fout.flush();
           
            // get answer message
            ans = fin.readLine();
            
            // if it is "open" the update
            if (ans.equals(RFIDCustom.kwOpen)) {
                update();
            }
           
            ftext.append("postupdate ARV = " + makeString.intArrayToString(ARV)+ "\n");
            ftext.append(ans + "\n");
            fMsgLabel.setText(ans);
            
            updateDisplay();
            
        } catch (IOException ex) {
            ftext.append("there was an exception: ans = " + ans + "\n");
            ex.printStackTrace();
        }
    }
    
    /**/public void resetIsXored() {
        for (int i=0; i<isXored.length; i++) {
            isXored[i] = false;
        }
    }
    
    private int[] encrypt() {
	keyEntry = n - (i - 1)%n -1;
        Random r = new Random();
	
        /* build indices array */
        boolean[] indicesflag = new boolean[n];
        for (int j=0; j<numOfEntries; j++) { indicesflag[j] = false; } // init indicesflag
        for (int j=0; j<numOfEntries; j++) {
            int index = r.nextInt(ARV.length);
            while (indicesflag[index]) {
                index = r.nextInt(ARV.length);
            }
            indicesflag[index] = true;
            indices[j] = index;
        }
        
        reducedLRV = RFIDCustom.createPseudoRandomArray((new Random()).nextInt(), numOfEntries, fdisplayBitNumber);
        // Y = (X=ARV[keyEntry] || indices || LRV)
        Y = RFIDCustom.concatArray(ARV[keyEntry], RFIDCustom.concatArray(indices, reducedLRV));
        
        ftext.append("keyEntry = " + keyEntry + "\n");
	ftext.append("preupdate reducedLRV = " + makeString.intArrayToString(ARV) + "\n");
        ftext.append("reducedLRV = " + makeString.intArrayToString(reducedLRV) + "\n");
        ftext.append("Y = " + makeString.intArrayToString(Y) + "\n");
        
        return Y;
    }
    
    private void update() {
	ARV[keyEntry] = 0;
	for (int j=0; j<indices.length; j++) {
            ARV[indices[j]] = ARV[indices[j]] ^ reducedLRV[j];
            /**/isXored[indices[j]] = true;
        }
	i++;
    }
    
    public void close() throws IOException {	// Close all I/O
        fout.close();
	fin.close();
	fclientSocket.close();
        ftext.append("close comunication\n");
    }
    
    public int[] getARV() {
        return ARV.clone();
    }

    /**/public boolean[] getIsXored() {
        return isXored.clone();
    }
     
}
