import java.awt.TextArea;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.StringTokenizer;

/////////////////////////////////////////////////////////////////////////////////
import javax.swing.JLabel;
import javax.swing.JList;

public class RFIDAP2UngenerlizedVerifier {

    private BufferedReader fin;	// stream form sender 
    private PrintWriter fout;	// stream to sender
    private ServerSocket fserverSocket;
    private Socket fclientSocket;
    private int fport;

    private int n;
    private int i;
    private int seed;
    public int[] ARV;
    int [] LRV;    
    public int[] prs;
    private int keyEntry;
    int[] Z;
	
    private Thread t;
    private int fdelayFactor;
    private int fdisplayType;
    private int fdisplayBitNumber;
    
    private TextArea ftext;
    private JList fARVList;
    private JList fLRVList;
    private JList fprsList;
    private JList fZList;
    private JLabel fSeedLabel;
    private JLabel fkeyEntryLabel;


    // IntListUpdater class
    class IntListUpdater implements Command {
        int[] farray;
        JList flist;
        
        public IntListUpdater(int[] array, JList list) {
            farray = array;
            flist = list;
        }
        
         public void doMy() {
            if (farray != null && flist != null)
                RFIDCustom.displayArray(flist, farray, fdisplayType, fdisplayBitNumber);
         }        
    }
    
    class StrLabelUndater implements Command {

        JLabel flabel;
        String fstr;
        
        public StrLabelUndater(String str, JLabel label) {
            flabel = label;
            fstr = str;
        }
        
        public void doMy() {
            if (fstr != null && flabel != null)
                flabel.setText(fstr);
        }
        
    }
    
    class VerifierDo implements Runnable {
        
        private void init() {
        try {
            ftext.append("initializing...\n");
            fserverSocket = new ServerSocket(fport);

            ftext.append("waiting for sender connection...\n");
            fclientSocket = fserverSocket.accept();
            ftext.append("accepted connection from sender at " + fclientSocket.getInetAddress() + ":" + fclientSocket.getPort() + "\n");

            ftext.append("initializing IO with sender\n");
            fin = new BufferedReader(new InputStreamReader(fclientSocket.getInputStream()));
            fout = new PrintWriter(fclientSocket.getOutputStream(), true);
            ftext.append("IO with sender is initialized\n");
        } catch (IOException ex) {
            ex.printStackTrace();
        }                    
    }
    
        private void update() {
            ARV[keyEntry] = 0;
            for (int j=0; j<ARV.length ; j++) {
                ARV[j] = ARV[j] ^ LRV[j];
            }
            i++;
        }
        
        public void updateDisplay() {
            Command[] cs = {new IntListUpdater(ARV, fARVList),
                            new StrLabelUndater(RFIDCustom.displayInt(seed, fdisplayType, fdisplayBitNumber), fSeedLabel),
                            new StrLabelUndater(RFIDCustom.displayInt(keyEntry, fdisplayType, fdisplayBitNumber), fkeyEntryLabel),                                                   
                            new IntListUpdater(/*reduced*/LRV, fLRVList),
                            new IntListUpdater(prs, fprsList),
                            new IntListUpdater(Z, fZList)
                           };
            
            RFIDDisplayDeamon dd = new RFIDDisplayDeamon(cs, fdelayFactor);
            dd.start();
        }

        private String decrypt(int[] Y) {
            String result = "";
            int X;

            // Y = (LRV || keyWord) xor prs
            keyEntry = n - (i - 1)%n -1;
            X = ARV[keyEntry];
            prs = RFIDCustom.createPseudoRandomArray(X, Y.length, fdisplayBitNumber);
            seed = X ^ seed;
            Z = RFIDCustom.xorArray(Y, prs);          
            
            System.arraycopy(Z, 0, LRV, 0, n);
            for (int j=n; j<Z.length; j++) {
                result = result + (char)Z[j];
            }
            
            ftext.append("prs = " + makeString.intArrayToString(prs)+ "\n");
            ftext.append("prs length = " + prs.length + "\n");
            ftext.append("LRV = " + makeString.intArrayToString(LRV)+ "\n");
            ftext.append("prev seed = " + (seed ^ X)+ "\n");
            ftext.append("new seed = " + seed+ "\n");      
            ftext.append("keyEntry = " + keyEntry+ "\n");
            ftext.append("X = " + X + "\n");

            return result;
        }//decrypt
        
        public void run() {
            String msg;
            String ans = null;
            boolean finish = false;

            init();

            try {
                while (!finish && (msg = fin.readLine()) != null) {
                    ftext.append(msg + "\n");

                    if (msg.equals("exit")) {
                        ans = "You are disconnected from reciver";
                        finish = true;
                    } else {
                        StringTokenizer st = new StringTokenizer(msg, ";");
                        try {
                            // convert msg to Y if it is in the right format <sizeOfMessage, Y[1], ..., LRV[sizeOfMessage-1]> 
                            int[] Y = new int[Integer.parseInt(st.nextToken())];
                            for (int j=0; j<Y.length; j++) {
                                int num = Integer.parseInt(st.nextToken());
                                ftext.append("Y[" + j + "] = " + num + "\n");
                                Y[j] = num;//Integer.parseInt(st.nextToken());                            
                            }
                            
                            // if decrypt Y is true then update 
                            if (decrypt(Y).equals(RFIDCustom.kwOpen)) {
                                ans = RFIDCustom.kwOpen;
                                update();
                            } else 
                                ans = RFIDCustom.kwDoNotOpen;
                            
                       } catch (Exception e) {
                            ftext.append("the message is not in valid format\n");
                            ans = RFIDCustom.kwDoNotOpen;
                       }
                    }//else

                    fout.println(ans);
                    
                    // update display
                    updateDisplay();
                    
                }//while
            } catch (IOException e) {
                try {
                    close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
                e.printStackTrace();
            }

            try {
                close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }

    }//run
     }
    
    public RFIDAP2UngenerlizedVerifier(int port, int[] FARV,
            int delayFactor, int displayType, int displayBitNumber,
            TextArea text, 
            JList ARVList, JList LRVList, JList prsList, JList ZList,
            JLabel SeedLabel, JLabel keyEntryLabel) {       
        fport = port;
        fin = null;
        fout = null;
        fserverSocket = null;
        fclientSocket = null;
        
        fdelayFactor = delayFactor;
        fdisplayType = displayType;
        fdisplayBitNumber = displayBitNumber;
        
        i = 1;
        seed = 0;
        ARV = FARV.clone();
        n = ARV.length;
        LRV = new int[n];
        
        ftext = text;
        fARVList = ARVList;
        fLRVList = LRVList;
        fprsList = prsList;
        fZList = ZList;
        fSeedLabel = SeedLabel;
        fkeyEntryLabel = keyEntryLabel;
        
        RFIDCustom.displayArray(fARVList, ARV, fdisplayType, fdisplayBitNumber);
        RFIDCustom.displayArray(fLRVList, LRV, fdisplayType, fdisplayBitNumber);
        
        t = new Thread(new VerifierDo());
        t.start();
    }    

    public void close() throws IOException {
        t.stop();
        fin.close();
        fout.close();
        fclientSocket.close();
        fserverSocket.close();
        ftext.append("close comunication\n");
    }
    

}
