import java.util.Random;

/* RFIDCustom class: a base class for RFID Sender & Reciver */
import javax.swing.DefaultListModel;
import javax.swing.JList;
public class RFIDCustom {
    
    
    // keywords consts
    public final static String kwOpen      = "Open";
    public final static String kwDoNotOpen = "DoNotOpen";
    public final static String keyWords[]  = {kwOpen, kwDoNotOpen};    
    
    // protocol type consts
    public final static int PROTOCOL_AP1_UNGENERLIZED = 0;
    public final static int PROTOCOL_AP1_GENERLIZED   = 1;
    public final static int PROTOCOL_AP2_UNGENERLIZED = 2;
    public final static int PROTOCOL_AP2_GENERLIZED   = 3;
    
    // display type consts 
    public final static int dtDec = 0;
    public final static int dtBin = 1;
    public final static int dtOct = 2;
    public final static int dtHex = 3;

    public static int[] xorArray(int[] src, int[] key) {
        int[] result = new int[src.length];
	for (int j=0; j<src.length; j++) {
		result[j] = src[j] ^ key[j];
	}
	return result;
    }
	
    public static int[] createPseudoRandomArray(int seed, int size, int bitNumber) {
	int[] result = new int[size];
        Random r = new Random(seed);

        for (int i=0; i<size; i++) {
            result[i] = r.nextInt(1 << bitNumber);
        }
        return result;
    }

    public static int[] concatArray(int[] A, int[] B) {
        int[] C = new int[A.length + B.length];
		
        System.arraycopy(A, 0, C, 0, A.length);
	System.arraycopy(B, 0, C, A.length, B.length);		 
	return C;
    }
    
    public static int[] concatArray(int number, int[] B) {
        int[] A = {number};
        		 
	return concatArray(A, B);
    }
    
    public static int[] concatArray(int[] A, String keyWord) {
	int[] B = new int[keyWord.length()];
	int[] C = new int[A.length + B.length];
		
	for (int i=0; i<keyWord.length(); i++) {
            B[i] = (int)keyWord.charAt(i);
	}
        System.arraycopy(A, 0, C, 0, A.length);
        System.arraycopy(B, 0, C, A.length, B.length);		 
        return C;
    }
	
    public static int[] copyArray(int[] src, int start, int length) {
		int[] result = new int[length];
		System.arraycopy(src, start, result, 0, length);
		return result;
	}

    private static double getRatio(int displayType) {
        switch (displayType) {
                case dtDec:
                    return 3.222;
                case dtBin:
                    return 1;
                case dtOct:
                    return 3;
                case dtHex:
                    return 4;
                default:
                    return 3.222;
            }
    }

    private static String fill(char c, int count) {
        char[] result = new char[count];
        for (int i=0; i<count; i++) {
            result[i] = c;
        }
        return new String(result);
    }    
    
    public static String displayInt(int num, int displayType, int bitNumber) {
        String str;
        int mask = -1 >>> (32 - bitNumber);
        int len = (int)Math.ceil(bitNumber/getRatio(displayType));
        int maskedEntry = mask & num;
        
        switch (displayType) {
                case dtDec:
                    str = Integer.toString(maskedEntry);
                    break;
                case dtBin:
                    str = Integer.toBinaryString(maskedEntry);
                    break;
                case dtOct:
                    str = Integer.toOctalString(maskedEntry);
                    break;
                case dtHex:
                    str = Integer.toHexString(maskedEntry);
                    break;
                default:
                    str = Integer.toString(maskedEntry);
                    break;
            }
        String result;
        if (str.length() - len >= 0)
            result = str.substring(str.length() - len, str.length());
        else
            result = fill('0', len - str.length()) +  str;
        return result;
    }
    
    public static void displayArray(JList list, int[] array, int displayType, int bitNumber) {
        DefaultListModel ml = new DefaultListModel();
        String str = null;
        // returns the value of (value | 0...01...1)
        int mask = -1 >>> (32 - bitNumber);
        int len = (int)Math.ceil(bitNumber/getRatio(displayType));
        
        for (int i=0; i<array.length; i++) {
            int maskedEntry = mask & array[i];
            
            switch (displayType) {
                case dtDec:
                    str = Integer.toString(maskedEntry);
                    break;
                case dtBin:
                    str = Integer.toBinaryString(maskedEntry);
                    break;
                case dtOct:
                    str = Integer.toOctalString(maskedEntry);
                    break;
                case dtHex:
                    str = Integer.toHexString(maskedEntry);
                    break;
                default:
                    str = Integer.toString(maskedEntry);
                    break;
            }                    
            String result;
            if (str.length() - len >= 0)
                result = str.substring(str.length() - len, str.length());
            else
                result = fill('0', len - str.length()) +  str;
            ml.addElement(result);
        }
        list.setModel(ml);
    }

}
