import java.awt.TextArea;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Random;
import java.util.StringTokenizer;

/* RFIDAdversary class: Simulate a RFID adversary */ 
import javax.swing.JTextArea;
public abstract class RFIDCustomAdversary {

    private boolean finish = false;
    private boolean finished = false;
    private int n;
    private int k;
    private boolean[] listeningVector;
    private int listeningIndex;
    
    private BufferedReader SenToAdrIn;	// stream form sender 
    private PrintWriter AdrToSenOut;	// stream to sender
    private ServerSocket SenToAdrServerSocket;
    private Socket SenToAdrClientSocket;
    private int fSenToAdrPort;

    private Socket clientSocket; 
    private BufferedReader in;
    private OutputStreamWriter out;
    private String fhost;
    private int fAdvToRecPort;   

    private TextArea ftext;
    private Thread t;
    

    class AdvVerInit implements Runnable {
        
        private void init() {            
            // Trying to connect to a socket and initialize an output stream
            try {
               clientSocket = new Socket(fhost, fAdvToRecPort); // host and port            
               //translate each character according to UTF-8.
               out = new OutputStreamWriter(clientSocket.getOutputStream(), "UTF-8");
            } catch (UnknownHostException e) {
                System.exit(1);
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }

            // Initialize an input stream
            try {
                in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }       

            ftext.append("Connected to server!\n");
            
            ftext.append("initializing...\n");
            try {
                SenToAdrServerSocket = new ServerSocket(fSenToAdrPort);
                ftext.append("waiting for sender connection...\n");

                SenToAdrClientSocket = SenToAdrServerSocket.accept();        
                ftext.append("accepted connection from sender at " + SenToAdrClientSocket.getInetAddress() + ":" + SenToAdrClientSocket.getPort() + "\n");

                SenToAdrIn = new BufferedReader(new InputStreamReader(SenToAdrClientSocket.getInputStream()));
                AdrToSenOut = new PrintWriter(SenToAdrClientSocket.getOutputStream(), true);
                
                ftext.append("IO with sender is initialized\n");
            } catch (IOException e) {
                e.printStackTrace();
            }//try
        }
        
        public void run() {
            String msg;
            String ans = null;
            
            init();

            try {
                // get msg from sender
                while (!finish && (msg = SenToAdrIn.readLine()) != null) {
                    ftext.append("RFID Adversary:\t got\t [" + msg + "]\n");

                    if (msg.equals("exit")) {
                        ans = "You are disconnected from adversary";
                        finish = true;
                    } else {

                        StringTokenizer st = new StringTokenizer(msg, ";");
                        try {
                            int[] Y = new int[Integer.parseInt(st.nextToken())];
                            for (int j=0; j<Y.length; j++) {
                                Y[j] = Integer.parseInt(st.nextToken());
                            }

                            // if listening then try to hack
                            if (listeningVector[listeningIndex]) {
                                ftext.append("listening to session\n");
                                if (hack(Y)) {
                                    ftext.append("RFID Adversary:\t [!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! SUCCEED TO BREAK [!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!]\n");
                                } else {
                                    ftext.append("RFID Adversary:\t [failed to break]\n");
                                }	        			   
                            } else {
                                ftext.append("not listening to session\n");
                            } // if
                            listeningIndex = (listeningIndex+1) % n;

                        } catch (Exception e) {
                            // msg is not in vaild format
                            ftext.append("RFID Adversary:\t [debug]\n");
                            ans = "";
                        }//try
                    }//else

                    // send msg to reciver
                    ftext.append("RFID Adversary:\t send to reciver \t [" + msg + "]\n");
                    out.write(msg + "\n");
                    out.flush();

                    // get msg form reciver
                    String RecToAdvMsg = in.readLine();
                    ftext.append("RFID Adversary:\t got form reciver \t [" + RecToAdvMsg + "]\n");

                    // send msg to sender
                    ans = RecToAdvMsg;

                    ftext.append("RFID Adversary:\t send to sender\t [" + ans + "]\n");
                    AdrToSenOut.println(ans);
                }

            } catch (IOException e) {
                ftext.append("RFID Adversary:\t Exception in processing\n");

                try {
                    close();
                } catch (IOException e1) {
                    ftext.append("RFID Adversary:\t Exception in IO while handling exception in processing\n");
                    e1.printStackTrace();
                }

                e.printStackTrace();
            } // try        

            try {
                close();
            } catch (IOException e1) {
                ftext.append("RFID Receiver:\t Exception in IO\n");
                e1.printStackTrace();
            }           
            finished = true;
        }
    
    }//AdvVerInit    

    public RFIDCustomAdversary(int k, int n, int SenToAdrPort, String host, int AdvToRecPort, TextArea text) {
	SenToAdrServerSocket = null;
	SenToAdrClientSocket = null;
	SenToAdrIn = null;
	AdrToSenOut = null;
	fSenToAdrPort = SenToAdrPort;
		
        clientSocket = null; 
        out = null;
        in = null;
        fhost = host;
	fAdvToRecPort = AdvToRecPort;
        
        ftext = text;
        
        this.k = k;
        this.n = n;
        this.listeningIndex = 0;
        this.listeningVector = new boolean[n];
        for (int i=0; i<n; i++) { this.listeningVector[i] = false; }
        Random r = new Random();
        boolean flag;
        for (int i=0; i<n-k; i++) {
            flag = true;
            while (flag) {
                int index = r.nextInt(this.listeningVector.length);
                if (!this.listeningVector[index]) {
                    this.listeningVector[index] = true;
                    flag = false;
                }
            }
        }
        
        t = new Thread(new AdvVerInit());
        t.start();
    }

    public abstract boolean hack(int[] Y);
    
    public void close() throws IOException {
        this.finish = true;
        
        if (finished) 
            return;
        else
            t.stop();
        
        SenToAdrIn.close();
        AdrToSenOut.close();
        SenToAdrClientSocket.close();
        SenToAdrServerSocket.close();
        /* ************* */
        this.out.close();
        this.in.close();
        clientSocket.close();
        ftext.append("close comunication\n");
    }
    
    public boolean userHack(String msg) {
        try {
            out.write(msg + "\n");
            out.flush(); 

            // get msg form reciver
            String ans = in.readLine();
            return ans.equals(RFIDCustom.kwOpen);
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

}
